/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls.anzeigenglobal;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.agzs.tls.AnzeigeAktorTls;
import de.bsvrz.ste.agzs.tls.AnzeigeTypUmsetzungTls;
import de.bsvrz.ste.agzs.tls.GlobalUmsetzung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.anzeige 
 * und die Abarbeitung der ber diesen Typ enpfangenen spezifischen Attributgruppen.
 * Die Abarbeitung erfolgt mit den versorgten Umsetzungsklassen, oder wenn keine spezielle versorgt, mit der Standardumsetzung.<br>
 * Die Standardumsetzungsklasse ist die Klasse 'zswzg.kex.tls.AnzeigeTypUmsetzungTlsDefault'.<br>
 * <br>
 * Abarbeitung: AnzeigenGlobal -> KExTls.<br>
 *
 * @version $Revision: 1.4 $ / $Date: 2014/02/13 16:29:51 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class Anzeige implements ClientReceiverInterface, ClientSenderInterface
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Konfigurationsobjekt vom Typ typ.anzeige.
   */
  private ConfigurationObject m_anzeige;
  /**
   * Datenverteilerverbindung.
   */
  private ClientDavInterface m_connection;
  /**
   * Datenmodell.
   */
  private DataModel m_dataModel;
  /**
   * AnzeigeAktorTls mit Key DEs.
   */
  private HashMap<String, AnzeigeAktorTls> m_listeAnzeigeAktorTlsKeyDes;
  /**
   * AnzeigeAktorTls mit Key Anzeige.
   */
  private HashMap<String, AnzeigeAktorTls> m_listeAnzeigeAktorTlsKeyAnzeige;
  /**
   * Defaultklasse fr die AnzeigeTypUmsetzungTls.
   */
  private String klassenNameDefault = "de.bsvrz.ste.agzs.tls.AnzeigeTypUmsetzungTlsDefault";
  /**
   * Kombinationen von AttributGruppe und Methode fr die Umsetzung der Anzeige.
   */
  private String[][] methoden =
  {
    {"atg.anzeigeEigenschaftSoll", "anzeigeEigenschaftSoll"},
    {"atg.anzeigeAutarkSoll", "anzeigeAutarkSoll"},
  };
  /**
   * Information, ob die Anzeige freie Grafik darstellen kann.
   */
  private boolean m_kannFreieGrafik = false;
  /**
   * Information, ob die Anzeige freien Text darstellen kann.
   */
  private boolean m_kannFreiText = false;
  /**
   * Information, ob die Daten zum Versand angemeldet sind.
   */
  private boolean m_datenversandAngemeldet = false;
  /**
   * Information, ob die Daten zum Empfang angemeldet sind.
   */
  private boolean m_datenempfangAngemeldet = false;
  /**
   * Instanz der Klasse AnzeigeQuerschnitt dem die Anzeige angehrt.
   */
  private AnzeigeQuerschnitt m_anzeigeQuerschnitt = null;
  /**
   * Liste der anzumeldenden Senken.
   */
  private final String[][] m_davToAnzeigeSenke = new String[][]
  {
    // Attributgruppe, Aspekt
		{"atg.anzeigeEigenschaftSoll", "asp.aqVorgabe"},
		{"atg.anzeigeAutarkSoll", "asp.aqVorgabe"},
  };


  /**
   * Konstruktor mit bergabe der notwendigen Parameter.
   * 
   * @param connection Datenverteilerverbindung.
   * @param anzeige Konfigurationsobjekt vom Typ typ.anzeige.
   * @param listeAnzeigeAktorTlsKeyDes HashMap mit den AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsKeyAnzeige HashMap mit den AnzeigeAktorTls mit Key Anzeige.
   */
  public Anzeige(ClientDavInterface connection, ConfigurationObject anzeige, HashMap<String, AnzeigeAktorTls> listeAnzeigeAktorTlsKeyDes, HashMap<String, AnzeigeAktorTls> listeAnzeigeAktorTlsKeyAnzeige)
  {
    AttributeGroup atgDaten;

    m_connection = connection;
    m_anzeige = anzeige;
    m_listeAnzeigeAktorTlsKeyDes = listeAnzeigeAktorTlsKeyDes;
    m_listeAnzeigeAktorTlsKeyAnzeige = listeAnzeigeAktorTlsKeyAnzeige;
    m_dataModel = m_connection.getDataModel();

//    m_debug.error("Anzeige " + m_anzeige.getPid());
    
    // Konfigurationsdaten holen
    
    atgDaten = m_dataModel.getAttributeGroup("atg.anzeige");
    
    Data daten = anzeige.getConfigurationData(atgDaten);
    
    SystemObject anzeigeTyp = daten.getReferenceValue("AnzeigeTyp").getSystemObject();
    
    if (anzeigeTyp != null)
    {
      atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeTyp");
      
      daten = anzeigeTyp.getConfigurationData(atgDaten);

      setKannFreieGrafik(daten.getScaledValue("KannFreieGrafik").getValueText().equals("Ja"));
      setKannFreiText(daten.getScaledValue("KannFreiText").getValueText().equals("Ja"));
    }
  }

  /**
   * Anmelden fr den Datenversand.<br>
   * Die Anmeldung wird nur durchgefhrt, wenn nicht angemeldet.<br>
   * 
   * @param sender Instanz des Senders.
   */
  protected void anmeldungDatenversand(ClientSenderInterface sender)
  {
    if (m_datenversandAngemeldet)
    {
      return;
    }
    
    m_datenversandAngemeldet = true;
    
    // Anzeige auf Datenversand anmelden
        
    AttributeGroup atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst");
    Aspect aspDaten = m_dataModel.getAspect("asp.aqZustand");
    DataDescription ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Anmeldung Quelle " + m_anzeige + " " + ddDaten);

    try
    {
      m_connection.subscribeSender(sender, m_anzeige, ddDaten, SenderRole.source());
    }
    catch (OneSubscriptionPerSendData e)
    {
      e.printStackTrace();

      System.exit(0);
    }
    
    // Anzeige auf Datenversand anmelden
    
    atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeAutarkIst");
    aspDaten = m_dataModel.getAspect("asp.aqZustand");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Anmeldung Quelle " + m_anzeige + " " + ddDaten);

    try
    {
      m_connection.subscribeSender(sender, m_anzeige, ddDaten, SenderRole.source());
    }
    catch (OneSubscriptionPerSendData e)
    {
      e.printStackTrace();

      System.exit(0);
    }
    m_datenversandAngemeldet = true;
  }

  /**
   * Abmelden vom Datenversand.<br>
   * Die Abmeldung wird nur durchgefhrt, wenn angemeldet.<br>
   * 
   * @param sender Instanz des Senders.
   */
  protected void abmeldungDatenversand(ClientSenderInterface sender)
  {
    if (!m_datenversandAngemeldet)
    {
      return;
    }
    
    m_datenversandAngemeldet = false;
    
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // Anzeige vom Datenversand anmelden
        
    atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst");
    aspDaten = m_dataModel.getAspect("asp.aqZustand");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Abmeldung " + m_anzeige + " " + ddDaten);

    m_connection.unsubscribeSender(sender, m_anzeige, ddDaten);
    
    m_datenversandAngemeldet = false;
  }

  /**
   * Anmelden fr den Datenempfang.<br>
   * Die Anmeldung wird nur durchgefhrt, wenn nicht angemeldet.<br>
   * 
   * @param empfaenger Instanz des Empfngers.
   */
  protected void anmeldungDatenempfang(ClientReceiverInterface empfaenger)
  {
    if (m_datenempfangAngemeldet)
    {
      return;
    }
    
    m_datenempfangAngemeldet = true;
    
    // Anzeige auf Datenempfang anmelden

    int anzahlAngemeldet = 0;
    
    for (int i = 0; i < m_davToAnzeigeSenke.length; i++)
    {
    	boolean angemeldet = senkeAnmeldenObjekt(empfaenger, m_anzeige, m_davToAnzeigeSenke[i][0], m_davToAnzeigeSenke[i][1]);
    	
    	if (angemeldet)
    	{
    		anzahlAngemeldet++;
    	}
    }
    
    if (anzahlAngemeldet != m_davToAnzeigeSenke.length)
    {
      m_debug.error("Nicht alle Anzeige-Senken konnten angemeldet werden");
    	
      System.exit(0);
    }
  }

  /**
   * Anmelden der bergebenen Parameter-Attributgruppen-Aspekt-Kombination fr die bergebene Objekt-Liste als Empfnger. 
   * 
   * @param instanz Empfnger der Daten.
   * @param objekt Objekt.
   * @param atg Attributgruppe der anzumeldenden Daten.
   * @param asp Aspekt der anzumeldenden Daten.
   */
  private boolean senkeAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, String atg, String asp)
  {
  	boolean ergebnis = false;
  	
    AttributeGroup atgParameter;
    Aspect aspParameter;
    DataDescription dd;

    m_debug.config("SenkeAnmeldenObjekt " + atg + " " + asp + " fr " + objekt);
    
    try
    {
      atgParameter = m_dataModel.getAttributeGroup(atg);
      aspParameter = m_dataModel.getAspect(asp);
    
      dd = new DataDescription(atgParameter, aspParameter);

      m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.drain());

      ergebnis = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.finest("Daten nicht zum Senden angemeldet " + e.getMessage());
    } 
    catch (NullPointerException e)
    {
      m_debug.finest("NullPointerException " + e.getMessage());
    }
    catch (IllegalStateException e)
    {
      m_debug.finest("IllegalStateException " + e.getMessage());
    }
    
    return ergebnis;
  }

    /**
   * Abmelden vom Datenempfang.<br>
   * Die Abmeldung wird nur durchgefhrt, wenn angemeldet.<br>
   * 
   * @param empfaenger Instanz des Empfngers.
   */
  protected void abmeldungDatenempfang(ClientReceiverInterface empfaenger)
  {
    if (!m_datenempfangAngemeldet)
    {
      return;
    }
    
    m_datenempfangAngemeldet = false;
    
    AttributeGroup atgDaten;
    Aspect aspDaten;
    DataDescription ddDaten;
    
    // Anzeige vom Datenempfang anmelden
    
    atgDaten = m_dataModel.getAttributeGroup("atg.anzeigeEigenschaftSoll");
    aspDaten = m_dataModel.getAspect("asp.aqVorgabe");
    ddDaten = new DataDescription(atgDaten, aspDaten);
    
    m_debug.config("Abmeldung " + m_anzeige.getPid() + " " + ddDaten);

    m_connection.unsubscribeReceiver(empfaenger, m_anzeige, ddDaten);
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientReceiverInterface#update(stauma.dav.clientside.ResultData[])
   */
  public void update(ResultData[] results)
  {
    for (ResultData result : results)
    {
//      System.out.println("Anzeige Result " + result);

      anzeigeVerarbeiten(result, new Long(0));
    }
  }

  /**
   * Verarbeiten der Befehle fr die Anzeige.
   * 
   * @param result Befehle fr die Anzeige.
   * @param zeitpunktUebergeordnet bergeordneter Schaltzeitpunkt.
   * @return Befehle fr das Teilmodell KExTls.
   */
  public ResultData anzeigeVerarbeiten(ResultData result, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;
    
//    System.out.println("Anzeige Result " + result);
      
    boolean datenVerarbeitet = false;
      
    SystemObject object = result.getObject();
      
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    Aspect         asp = result.getDataDescription().getAspect();

    AnzeigeAktorTls anzeigeAktorUms = (AnzeigeAktorTls)m_listeAnzeigeAktorTlsKeyAnzeige.get(object.getPid());
      
    if (anzeigeAktorUms != null)
    {
      m_debug.fine("Umsetzung " + anzeigeAktorUms.getAnzeigeTypUmsetzung().getPid() + " " + result.toString());
//      m_debug.error("Umsetzung " + anzeigeAktorUms.getAnzeigeTypUmsetzung().getPid() + " " + result.toString());
  
      AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktorUms.getAnzeigeTypUmsetzung();
        
      int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();
  
      for (int i = 0; i < anzahlGlobalUmsetzung; i++)
      {
        GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(i);
          
        if (globalUmsetzung.getUmsetzungsModul() != null)
        {
        	String klassenName = globalUmsetzung.getUmsetzungsModul();
        	
        	klassenName = klassenName.replace("zswzg.kex.", "de.bsvrz.ste.agzs.");
        	
          ergebnis = umsetzungAbarbeiten(klassenName, result, zeitpunktUebergeordnet, object, atg, asp, anzeigeTypUmsetzungTls, i);

          if (ergebnis != null)
          {
            datenVerarbeitet = true;
            
            break;
          }
        }
      }
        
      if (!datenVerarbeitet)
      {
        ergebnis = umsetzungAbarbeiten(klassenNameDefault, result, zeitpunktUebergeordnet, object, atg, asp, anzeigeTypUmsetzungTls, -1);
      }
    }
    
    return ergebnis;
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientSenderInterface#dataRequest(stauma.dav.configuration.interfaces.SystemObject, stauma.dav.clientside.DataDescription, byte)
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
  }

  /* (non-Javadoc)
   * @see stauma.dav.clientside.ClientSenderInterface#isRequestSupported(stauma.dav.configuration.interfaces.SystemObject, stauma.dav.clientside.DataDescription)
   */
  public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
  {
    return false;
  }

  /**
   * Methode fr die Abarbeitung der Umsetzung fr die Anzeige.
   * 
   * @param klassenName Name der Klasse fr die Umsetzung.
   * @param result Eingangsdaten.
   * @param zeitpunktUebergeordnet bergeordneter Schaltzeitpunkt.
   * @param object Anzeige
   * @param atg AttributGruppe der Eingangsdaten.
   * @param asp Aspekt der Eingangsdaten.
   * @param anzeigeTypUmsetzungTls AnzeigeTypUmsetzungTls fr diese Anzeige.
   * @param indexUmsetzung Index der Umsetzung.
   * @return Befehle fr das Teilmodell KExTls.
   */
  public ResultData umsetzungAbarbeiten(String klassenName, ResultData result, Long zeitpunktUebergeordnet, SystemObject object, AttributeGroup atg, Aspect asp, AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int indexUmsetzung)
  {
    ResultData ergebnis = null;
    Class klasse = null;
    String aktuelleMethode = null;

    m_debug.fine("anzeigeUmsetzungAbarbeiten " + klassenName + " object " + object + " result " + result);

    try
    {
      klasse = Class.forName(klassenName);

//      System.out.println("klasse " + klasse);
      
      Class[] formaleParameter = new Class[4];
      formaleParameter[0] = HashMap.class;
      formaleParameter[1] = HashMap.class;
      formaleParameter[2] = boolean.class;
      formaleParameter[3] = boolean.class;
    
      Constructor construktor = klasse.getConstructor(formaleParameter);
      Object ret = construktor.newInstance(new Object[]{m_listeAnzeigeAktorTlsKeyDes, m_listeAnzeigeAktorTlsKeyAnzeige, isKannFreieGrafik(), isKannFreiText()});

      Method methode2 = null;
      
      for (int i = 0; i < methoden.length; i++)
      {
        if (atg.getPid().equals(methoden[i][0]))
        {
          aktuelleMethode = methoden[i][1];
          
//          System.out.println("methode " + aktuelleMethode);
          
          methode2 = klasse.getMethod(aktuelleMethode, new Class[]{ResultData.class, Long.class});
          
          break;
        }
      }
      
      if (methode2 != null)
      {
        ergebnis = (ResultData)methode2.invoke(ret, new Object[]{result, zeitpunktUebergeordnet});
      }
    }
    catch (ClassNotFoundException e)
    {
      m_debug.error("Klasse '" + klasse + "' fr das Abarbeiten der Anzeigenumsetzung konnte nicht geladen werden.");
      
      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      m_debug.error("Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "' existiert nicht.");

      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      m_debug.error("InvocationTargetException");

      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      m_debug.error("Fehler beim Zugriff auf die Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      m_debug.error("Fehler bei der Instanziierung der Methode '" + aktuelleMethode + "()' in der Klasse '" + klasse + "'.");

      e.printStackTrace();
    }

    return ergebnis;
  }

  /**
   * Holen der Information, ob die Anzeige freie Grafik darstellen kann.
   *  
   * @return <code>true</code>, wenn die Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   */
  public boolean isKannFreieGrafik()
  {
    return m_kannFreieGrafik;
  }

  /**
   * Setzen der Information, ob die Anzeige freie Grafik darstellen kann.
   *  
   * @param kannFreieGrafik <code>true</code>, wenn die Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   */
  public void setKannFreieGrafik(boolean kannFreieGrafik)
  {
    m_kannFreieGrafik = kannFreieGrafik;
  }

  /**
   * Holen der Information, ob die Anzeige freien Text darstellen kann.
   *  
   * @return <code>true</code>, wenn die Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public boolean isKannFreiText()
  {
    return m_kannFreiText;
  }

  /**
   * Setzen der Information, ob die Anzeige freien Text darstellen kann.
   *  
   * @param kannFreiText <code>true</code>, wenn die Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public void setKannFreiText(boolean kannFreiText)
  {
    m_kannFreiText = kannFreiText;
  }

  /**
   * Holen des AnzeigeQuerschnitt, dem die Anzeige angehrt.
   * 
   * @return AnzeigeQuerschnitt, dem die Anzeige angehrt.
   */
  public AnzeigeQuerschnitt getAnzeigeQuerschnitt()
  {
    return m_anzeigeQuerschnitt;
  }

  /**
   * Setzen des AnzeigeQuerschnitt, dem die Anzeige angehrt.
   * 
   * @param anzeigeQuerschnitt AnzeigeQuerschnitt, dem die Anzeige angehrt.
   */
  public void setAnzeigeQuerschnitt(AnzeigeQuerschnitt anzeigeQuerschnitt)
  {
    m_anzeigeQuerschnitt = anzeigeQuerschnitt;
  }
  
  /**
   * Holen des Empfngers fr die Befehle fr die Anzeige.
   * 
   * @return Empfnger fr die Befehle fr die Anzeige.
   */
  public ClientReceiverInterface getEmpfaenger()
  {
    ClientReceiverInterface empfaenger = this;
    
    if (getAnzeigeQuerschnitt() != null)
    {
      empfaenger = getAnzeigeQuerschnitt().getEmpfaenger();
    }
    
    return empfaenger;
  }

  /**
   * Holen des Senders fr die Befehle fr die DE.
   * 
   * @return Senders fr die Befehle fr die DE.
   */
  public ClientSenderInterface getSender()
  {
    ClientSenderInterface sender = this;
    
    return sender;
  }

  /**
   * Holen der Pid der Anzeige.
   * 
   * @return Pid der Anzeige.
   */
  public String getPid()
  {
    return m_anzeige.getPid();
  }

  /**
   * Anmelden zum Datenempfang fr den Empfnger.
   */
  public void anmeldungDatenempfang()
  {
    anmeldungDatenempfang(getEmpfaenger());
  }
  /**
   * Anmelden zum Datenversand fr den Sender.
   */
  public void anmeldungDatenversand()
  {
    anmeldungDatenversand(getSender());
  }
  
  /**
   * Senden von Daten ber den Sender.
   * 
   * @param resultData zu versendende Daten.
   * @param zeitpunkt bergeordneter Schaltzeitpunkt.
   * @return <code>true</code>.
   */
  public boolean datenSenden(ResultData resultData, long zeitpunkt)
  {
    if (zeitpunkt != 0)
    {
      resultData.setDataTime(zeitpunkt);
    }
    
    try
    {
      m_connection.sendData(resultData);
    }
    catch (DataNotSubscribedException e)
    {
      e.printStackTrace();
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      e.printStackTrace();
    }
    
    return true;
  }

  /**
   * Holen der HashMap mit den AnzeigeAktorTls mit Key Anzeige.
   * 
   * @return HashMap mit den AnzeigeAktorTls mit Key Anzeige.
   */
  public HashMap<String, AnzeigeAktorTls> getListeAnzeigeAktorTlsKeyAnzeige()
  {
    return m_listeAnzeigeAktorTlsKeyAnzeige;
  }
}